-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000-2002                     --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@unican.es                     --
--          Jose Javier Gutierrez  gutierjj@unican.es                --
--          Jose Carlos Palencia   palencij@unican.es                --
--          Jose Maria Drake       drakej@unican.es                  --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Ada.Text_IO, Mast.Drivers, Mast.Timers;
package Mast.Processing_Resources.FP is

   -----------------------
   -- Processor
   -----------------------

   type Processor is abstract new Processing_Resource with private;

   type Processor_Ref is access Processor'Class;

   -----------------------
   -- Network
   -----------------------

   type Network is abstract new Processing_Resource with private;

   type Network_Ref is access Network'Class;

   type Driver_Iteration_Object is private;

   procedure Add_Driver
     (Net : in out Network;
      The_Driver : MAST.Drivers.Driver_Ref);

   function Num_Of_Drivers
     (Net : Network)
     return Natural;

   procedure Rewind_Drivers
     (Net : in Network;
      Iterator : out Driver_Iteration_Object);

   procedure Get_Next_Driver
     (Net : in Network;
      The_Driver : out MAST.Drivers.Driver_Ref;
      Iterator : in out Driver_Iteration_Object);

   ---------------------------
   -- Fixed_Priority_Processor
   ---------------------------

   type Fixed_Priority_Processor is new Processor with private;

   procedure Set_Max_Interrupt_Priority
     (Res : in out Fixed_Priority_Processor;
      The_Priority : Priority);
   function Max_Interrupt_Priority
     (Res : Fixed_Priority_Processor)
     return Priority;

   procedure Set_Min_Interrupt_Priority
     (Res : in out Fixed_Priority_Processor;
      The_Priority : Priority);
   function Min_Interrupt_Priority
     (Res : Fixed_Priority_Processor)
     return Priority;

   function Max_Any_Priority (Res : Fixed_Priority_Processor)
                             return Priority;

   procedure Set_Worst_Context_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Worst_Context_Switch : Time);
   procedure Set_Avg_Context_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Avg_Context_Switch : Time);
   procedure Set_Best_Context_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Best_Context_Switch : Time);
   function Worst_Context_Switch
     (CPU : Fixed_Priority_Processor) return Time;
   function Avg_Context_Switch
     (CPU : Fixed_Priority_Processor) return Time;
   function Best_Context_Switch
     (CPU : Fixed_Priority_Processor) return Time;

   procedure Set_Worst_ISR_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Worst_ISR_Switch : Time);
   procedure Set_Avg_ISR_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Avg_ISR_Switch : Time);
   procedure Set_Best_ISR_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Best_ISR_Switch : Time);
   function Worst_ISR_Switch
     (CPU : Fixed_Priority_Processor) return Time;
   function Avg_ISR_Switch
     (CPU : Fixed_Priority_Processor) return Time;
   function Best_ISR_Switch
     (CPU : Fixed_Priority_Processor) return Time;

   procedure Set_System_Timer
     (CPU : in out Fixed_Priority_Processor;
      The_System_Timer : Timers.System_Timer_Ref);

   function The_System_Timer
     (CPU : Fixed_Priority_Processor)
     return Timers.System_Timer_Ref;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Fixed_Priority_Processor;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   function Clone
     (Res : Fixed_Priority_Processor)
     return  Processing_Resource_Ref;

   ----------------
   -- Network
   ----------------

   type Fixed_Priority_Network is
     new Network with private;

   procedure Set_Transmission_Mode
     (Net  : in out Fixed_Priority_Network;
      The_Transmission_Mode : Transmission_Kind);
   function Transmission_Mode
     (Net  : Fixed_Priority_Network) return Transmission_Kind;

   procedure Set_Max_Blocking
     (Net  : in out Fixed_Priority_Network;
      The_Max_Blocking : Time);
   function Max_Blocking
     (Net  : Fixed_Priority_Network) return Time;

   procedure Set_Max_Packet_Transmission_Time
     (Net  : in out Fixed_Priority_Network;
      The_Max_Transmission_Time : Time);
   function Max_Packet_Transmission_Time
     (Net  : Fixed_Priority_Network) return Time;

   procedure Set_Min_Packet_Transmission_Time
     (Net  : in out Fixed_Priority_Network;
      The_Min_Transmission_Time : Time);
   function Min_Packet_Transmission_Time
     (Net  : Fixed_Priority_Network) return Time;

   procedure Set_Packet_Worst_Overhead
     (Net  : in out Fixed_Priority_Network;
      The_Worst_Overhead : Time);
   function Packet_Worst_Overhead
     (Net  : Fixed_Priority_Network)
     return Time;

   procedure Set_Packet_Avg_Overhead
     (Net  : in out Fixed_Priority_Network;
      The_Avg_Overhead : Time);
   function Packet_Avg_Overhead
     (Net  : Fixed_Priority_Network)
     return Time;

   procedure Set_Packet_Best_Overhead
     (Net  : in out Fixed_Priority_Network;
      The_Best_Overhead : Time);
   function Packet_Best_Overhead
     (Net  : Fixed_Priority_Network)
     return Time;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Fixed_Priority_Network;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   function Clone
     (Res : Fixed_Priority_Network)
     return  Processing_Resource_Ref;

private

   type Processor is abstract new Processing_Resource with null record;

   type Driver_Iteration_Object is new Drivers.Lists.Iteration_Object;

   type Network is abstract new Processing_Resource with record
      List_Of_Drivers : Drivers.Lists.List;
   end record;

   type Fixed_Priority_Processor is
     new Processor with record
        Worst_Context_Switch,
        Avg_Context_Switch,
        Best_Context_Switch : Time:=0.0;
        Worst_ISR_Switch,
        Avg_ISR_Switch,
        Best_ISR_Switch : Time:=0.0;
        Max_Interrupt_Priority : Priority:=Priority'Last;
        Min_Interrupt_Priority : Priority:=Priority'First;
        The_System_Timer : Timers.System_Timer_Ref;
   end record;

   type Fixed_Priority_Network is
     new Network with record
        Transmission_Mode : Transmission_Kind:=Half_Duplex;
        Packet_Worst_Overhead,
        Packet_Avg_Overhead,
        Packet_Best_Overhead : Time:= 0.0;
        Max_Blocking : Time:=0.0;
        Max_Packet_Transmission_Time,
        Min_Packet_Transmission_Time: Time :=Large_Time;
   end record;

end Mast.Processing_Resources.FP;
